/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <joerg@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.        Joerg Wunsch
 * ----------------------------------------------------------------------------
 */
#include <stdint.h>
#include <util/atomic.h>

static uint8_t
SPITransfer(uint8_t d)
{
	USIDR = d;
	USISR = _BV(USIOIF);
	do {
		USICR = _BV(USIWM0) | _BV(USICS1) |
			_BV(USICLK) | _BV(USITC);
	} while ((USISR & _BV(USIOIF)) == 0);
	return USIDR;
}

#include "radioif.h"

uint8_t
rf230_regrd(uint8_t regno)
{
	uint8_t rv;

	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		PORTA &= ~_BV(7);
		(void)SPITransfer(0x80 | regno);
		rv = SPITransfer(0);
		PORTA |= _BV(7);
	}
	return rv;
}

void
rf230_regwr(uint8_t regno, uint8_t val)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		PORTA &= ~_BV(7);
		(void)SPITransfer(0xc0 | regno);
		(void)SPITransfer(val);
		PORTA |= _BV(7);
	}
}

void
rf230_frmwr(uint8_t len, uint8_t *buf)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		PORTA &= ~_BV(7);
		(void)SPITransfer(0x60);
		(void)SPITransfer(len);
		len &= 0x7F;
		while (len-- != 0)
			(void)SPITransfer(*buf++);
		PORTA |= _BV(7);
	}
}

void
rf230_frmrd(uint8_t *len, uint8_t *buf)
{
	uint8_t i;

	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		PORTA &= ~_BV(7);
		(void)SPITransfer(0x20);
		*len = i = SPITransfer(0);
		i &= 0x7F;
		while (i-- != 0)
			*buf++ = SPITransfer(0);
		PORTA |= _BV(7);
	}
}
