/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <joerg@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.        Joerg Wunsch
 * ----------------------------------------------------------------------------
 */

#include <stdint.h>

uint8_t rf230_regrd(uint8_t regno);
void rf230_regwr(uint8_t regno, uint8_t val);
void rf230_frmrd(uint8_t *len, uint8_t *buf);
void rf230_frmwr(uint8_t len, uint8_t *buf);

#define TRX_STATUS 0x01
#define GET_TRX_STATUS() (rf230_regrd(TRX_STATUS) & 0x1f)

typedef enum {
	TRX_STATUS_P_ON,
	TRX_STATUS_BUSY_RX = 1,
	TRX_STATUS_BUSY_TX = 2,
	TRX_STATUS_RX_ON = 6,
	TRX_STATUS_TRX_OFF = 8,
	TRX_STATUS_PLL_ON = 9,
	TRX_STATUS_SLEEP = 0x0F,
	TRX_STATUS_BUSY_RX_AACK = 0x11,
	TRX_STATUS_BUSY_TX_ARET = 0x12,
	TRX_STATUS_RX_AACK_ON = 0x16,
	TRX_STATUS_TX_ARET_ON = 0x19,
	TRX_STATUS_RX_ON_NOCLK = 0x1C,
	TRX_STATUS_RX_AACK_ON_NOCLK = 0x1D,
	TRX_STATUS_BUSY_RX_AACK_NOCLK = 0x1E,
	TRX_STATUS_STATE_TRANSITION_IN_PROGRESS = 0x1F
} __attribute__((packed)) trx_status_t;

#define TRX_STATE 0x02
#define GET_TRAC_STATUS() ((rf230_regrd(TRX_STATE) & 0xe0) >> 5)
#define SET_TRX_CMD(cmd) (rf230_regwr(TRX_STATE, cmd))

typedef enum {
	TRAC_SUCCESS,
	TRAC_SUCCESS_DATA_PENDING = 1,
	TRAC_CHANNEL_ACCESS_FAILURE = 3,
	TRAC_NO_ACK = 5,
	TRAC_INVALID = 7
} __attribute__((packed)) trac_state_t;
typedef enum {
	TRX_CMD_NOP,
	TRX_CMD_TX_START = 2,
	TRX_CMD_FORCE_TRX_OFF = 3,
	TRX_CMD_RX_ON = 6,
	TRX_CMD_TRX_OFF = 8,
	TRX_CMD_TX_ON = 9,
	TRX_CMD_RX_AACK_ON = 0x16,
	TRX_CMD_TX_ARET_ON = 0x19
} __attribute__((packed)) trx_cmd_t;

#define PHY_TX_PWR 0x05
#define SET_TX_PWR(pwr) (rf230_regwr(PHY_TX_PWR, (rf230_regrd(PHY_TX_PWR) & ~0x0F) | (pwr)))
#define SET_AUTO_CRC_ON(on) (rf230_regwr(PHY_TX_PWR, (rf230_regrd(PHY_TX_PWR) & ~0x80) | ((on)? 0x80: 0)))

typedef enum {
	TX_PWR_P3_0 = 0,	/* +3.0 dBm */
	TX_PWR_P2_6 = 1,	/* +2.6 dBm */
	TX_PWR_P2_1 = 2,	/* +2.1 dBm */
	TX_PWR_P1_6 = 3,	/* +1.6 dBm */
	TX_PWR_P1_1 = 4,	/* +1.1 dBm */
	TX_PWR_P0_5 = 5,	/* +0.5 dBm */
	TX_PWR_N0_2 = 6,	/* -0.2 dBm */
	TX_PWR_N1_2 = 7,	/* -1.2 dBm */
	TX_PWR_N2_2 = 8,	/* -2.2 dBm */
	TX_PWR_N3_2 = 9,	/* -3.2 dBm */
	TX_PWR_N4_2 = 10,	/* -4.2 dBm */
	TX_PWR_N5_2 = 11,	/* -5.2 dBm */
	TX_PWR_N7_2 = 12,	/* -7.2 dBm */
	TX_PWR_N9_2 = 13,	/* -9.2 dBm */
	TX_PWR_N12_2 = 14,	/* -12.2 dBm */
	TX_PWR_N17_2 = 15	/* -17.2 dBm */
} __attribute__((packed)) tx_pwr_t;


#define PHY_RSSI 0x06
#define GET_RX_CRC_VALID() (rf230_regrd(PHY_RSSI) & 0x80)
#define GET_RSSI() (rf230_regrd(PHY_RSSI) & 0x1f)

#define PHY_ED_LEVEL 0x07
#define GET_ED_LEVEL() (rf230_regrd(PHY_ED_LEVEL))

#define PHY_CC_CCA 0x08
#define SET_CHANNEL(chan) (rf230_regwr(PHY_CC_CCA, (rf230_regrd(PHY_CC_CCA) & ~0x1F) | (chan)))

#define IRQ_MASK 0x0E
#define IRQ_STATUS 0x0F
typedef enum {
        PLL_LOCK = 1,
        PLL_UNLOCK = 2,
        RX_START = 4,
        TRX_END = 8,
        TRX_UR = 64,
        BAT_LOW = 128
} __attribute__((packed)) irq_bits;

#define PART_NUM 0x1C
#define GET_PART_NUM() (rf230_regrd(PART_NUM))

#define VERSION_NUM 0x1D
#define GET_VERSION_NUM() (rf230_regrd(VERSION_NUM))

#define MAN_ID_0 0x1E
#define GET_MANID0() (rf230_regrd(MAN_ID_0))

#define MAN_ID_1 0x1F
#define GET_MANID1() (rf230_regrd(MAN_ID_1))

#define SHORT_ADDR_0 0x20
#define SHORT_ADDR_1 0x21
#define PAN_ID_0     0x22
#define PAN_ID_1     0x23
#define IEEE_ADDR_0  0x24
#define IEEE_ADDR_1  0x25
#define IEEE_ADDR_2  0x26
#define IEEE_ADDR_3  0x27
#define IEEE_ADDR_4  0x28
#define IEEE_ADDR_5  0x29
#define IEEE_ADDR_6  0x2A
#define IEEE_ADDR_7  0x2B

#define XAH_CTRL 0x2C
#define SET_MAX_FRAME_RETRIES(n) \
(rf230_regwr(XAH_CTRL, (rf230_regrd(XAH_CTRL) & ~0x70) | ((n) << 4)))
#define SET_MAX_CSMA_RETRIES(n) \
(rf230_regwr(XAH_CTRL, (rf230_regrd(XAH_CTRL) & ~0x07) | (n)))

#define CSMA_SEED_0  0x2D
#define CSMA_SEED_1  0x2E
#define SET_MIN_BE(n) \
(rf230_regwr(CSMA_SEED_1, (rf230_regrd(CSMA_SEED_1) & ~0xc0) | ((n) << 6)))
#define SET_I_AM_COORD(on) \
(rf230_regwr(CSMA_SEED_1, (rf230_regrd(CSMA_SEED_1) & ~0x08) | ((on)? 0x08: 0)))

#define SET_CSMA_SEED(n) \
do { \
rf230_regwr(CSMA_SEED_1, (rf230_regrd(CSMA_SEED_1) & ~0x07) | (((n) >> 8) & 7)); \
rf230_regwr(CSMA_SEED_0, (n) & 0xFF); \
} while (0)
